searchData={"items":[{"type":"module","title":"Formulae","doc":"A set of functions to deal with analytical formulae.\n\nThe typical way of using this module would be to call `Formulae.compile/1`\non the binary representing the string.\n\n```elixir\niex|1 ▶ f = Formulae.compile \"a + :math.sin(3.14 * div(b, 2)) - c\"\n\n#ℱ<[\n  sigil: \"~F[a + :math.sin(3.14 * div(b, 2)) - c]\",\n  eval: &:\"Elixir.Formulae.a + :math.sin(3.14 * div(b, 2)) - c\".eval/1,\n  formula: \"a + :math.sin(3.14 * div(b, 2)) - c\",\n  guard: nil,\n  module: :\"Elixir.Formulae.a + :math.sin(3.14 * div(b, 2)) - c\",\n  variables: [:a, :b, :c],\n  options: [defaults: [], imports: [:...], evaluator: :function, alias: nil]\n]>\n```\n\nNow the formula is compiled and might be invoked by calling `Formulae.eval/2`\npassing a formula _and_ bindings. First call to `eval/2` would lazily compile\nthe module if needed.\n\n```elixir\niex|2 ▶ f.eval.(a: 3, b: 4, c: 2)\n0.9968146982068622\n```\n\nWhether one needs to use external modules in formulas, these modules\nmust be explicitly imported via `imports: [Mod1, Mod2]`. In case of clash\nagainst `Kernel` functions, the latter might be “unimported” explicitly.\n\n```elixir\niex|3 ▶ Formulae.compile(\"div(100, 2)\", imports: [Decimal], unimports: [div: 2])\nDecimal.new(\"50\")\n```\n\nThe formulae might be curried.\n\n```elixir\niex|4 ▶ Formulae.curry(f, a: 3, b: 4)\n\n#ℱ<[\n  sigil: \"~F[3 + :math.sin(3.14 * div(4, 2)) - c]\",\n  eval: &:\"Elixir.Formulae.3 + :math.sin(3.14 * div(4, 2)) - c\".eval/1,\n  formula: \"3 + :math.sin(3.14 * div(4, 2)) - c\",\n  guard: nil,\n  module: :\"Elixir.Formulae.3 + :math.sin(3.14 * div(4, 2)) - c\",\n  variables: [:c],\n  options: [defaults: [], imports: [:...], evaluator: :function, alias: nil]\n]>\n```\n\nSince `v0.10.0` there is an ability to pass `defaults` via `options`.\n\n_Examples:_\n\n    iex> \"z + t\" |> Formulae.compile(defaults: [t: 5]) |> Formulae.eval(t: 10, z: 3)\n    13\n    iex> \"z + t\" |> Formulae.compile(defaults: [t: 5]) |> Formulae.eval(z: 3)\n    8","ref":"Formulae.html"},{"type":"function","title":"Formulae.bindings?/3","doc":"Returns the binding this formula requires.","ref":"Formulae.html#bindings?/3"},{"type":"function","title":"Examples - Formulae.bindings?/3","doc":"iex> \"a > 5\" |> Formulae.bindings?()\n    ~w|a|a\n\n    iex> \":math.sin(a / (3.14 * b)) > c\" |> Formulae.bindings?([], imports: [:math])\n    ~w|a b c|a\n\n    iex> \"a + b * 4 - :math.pow(c, 2) / d > 1.0 * e\" |> Formulae.bindings?([], imports: :math)\n    ~w|a b c d e|a","ref":"Formulae.html#bindings?/3-examples"},{"type":"function","title":"Formulae.check/3","doc":"Revalidates the formula with bindings given. Returns true if the formula\nstrictly evaluates to `true`, `false` otherwise. Compiles the formula\nbefore evaluation if needed.","ref":"Formulae.html#check/3"},{"type":"function","title":"Formulae.combinations/2","doc":"Generated clauses for `n ∈ [1..42]` to be used with dynamic number","ref":"Formulae.html#combinations/2"},{"type":"function","title":"Formulae.compile/2","doc":"Compiles the formula into module.\n\nThe allowed imports must be specified explicitly with `imports: :all` or\n  a list of allowed imports `imports: [DateTime, Range]`.\n\n_Examples:_\n\n    iex> f = Formulae.compile(\"rem(a, 5) - b == 0\")\n    iex> f.formula\n    \"rem(a, 5) - b == 0\"\n    iex> f.variables\n    [:a, :b]\n    iex> f.module\n    :\"Elixir.Formulae.rem(a, 5) - b == 0\"\n    iex> f.module.eval(a: 12, b: 2)\n    true\n\n    iex> f = Formulae.compile(\"rem(a, 5) + b == a\")\n    iex> f.variables\n    [:a, :b]\n    iex> f.eval.(a: 7, b: 5)\n    true\n    iex> f.eval.(a: 7, b: 0)\n    false","ref":"Formulae.html#compile/2"},{"type":"function","title":"Formulae.compiled?/2","doc":"Checks whether the formula was already compiled into module.\n\nTypically one does not need to call this function, since this check would be\nnevertheless transparently performed before the evaluation.\n\n_Examples:_\n\n    iex> Formulae.compiled?(\"foo > 42\")\n    false\n    iex> Formulae.compile(\"foo > 42\")\n    iex> Formulae.compiled?(\"foo > 42\")\n    true","ref":"Formulae.html#compiled?/2"},{"type":"function","title":"Formulae.curry/3","doc":"Curries the formula by substituting the known bindings into it.","ref":"Formulae.html#curry/3"},{"type":"function","title":"Example - Formulae.curry/3","doc":"iex> Formulae.curry(\"(temp - foo * 4) > speed / 3.14\", temp: 7, speed: 3.14).formula\n    \"7 - foo * 4 > 3.14 / 3.14\"","ref":"Formulae.html#curry/3-example"},{"type":"function","title":"Formulae.ensure_compiled/2","doc":"Checks whether the formula was already compiled into module.\nSimilar to `compiled?/1`, but returns what `Code.ensure_compiled/1` returns.\n\nTypically one does not need to call this function, since this check would be\nnevertheless transparently performed before the evaluation.\n\n_Examples:_\n\n    iex> Formulae.ensure_compiled(\"bar > 42\")\n    {:error, :nofile}\n    iex> Formulae.compile(\"bar > 42\")\n    iex> Formulae.ensure_compiled(\"bar > 42\")\n    {:module, :\"Elixir.Formulae.bar > 42\"}","ref":"Formulae.html#ensure_compiled/2"},{"type":"function","title":"Formulae.eval/3","doc":"Evaluates the formula returning the result back.\n\n_Examples:_\n\n    iex> Formulae.eval(\"rem(a, 5) + rem(b, 4) == 0\", a: 20, b: 20)\n    true\n    iex> Formulae.eval(\"rem(a, 5) == 0\", a: 21)\n    false\n    iex> Formulae.eval(\"rem(a, 5) + rem(b, 4)\", a: 21, b: 22)\n    3\n    iex> Formulae.eval(\"rem(a, 5) == b\", [a: 8], defaults: [b: 3])\n    true\n    iex> Formulae.eval(\"rem(a, 5) == c\", [a: 8, c: 3], defaults: [b: 3])\n    true\n    iex> Formulae.eval(\"to_integer(s) == i\", [s: \"42\", i: 42], imports: [String])\n    true\n\nBinary input is deprecated, create a formula explicitly with `Formulae.compile/2`\n  and then pass it as the first argument to `eval/2`\".\n\nThe call to `eval/3` would compile the formulae with default options.","ref":"Formulae.html#eval/3"},{"type":"function","title":"Formulae.eval!/3","doc":"Evaluates the formula returning the result back; throws in a case of unseccessful processing.\n\n_Examples:_\n\n    iex> Formulae.eval!(\"rem(a, 5) == 0\", a: 20)\n    true\n    iex> Formulae.eval!(\"rem(a, 5) == 0\")\n    ** (Formulae.RunnerError) Formula ~F[rem(a, 5) == 0] failed to run (compile): [:missing_arguments], wrong or incomplete evaluator call: [given_keys: [], expected_keys: [:a]].","ref":"Formulae.html#eval!/3"},{"type":"function","title":"Formulae.evaluate/3","doc":"Evaluates normalized representation of formula.","ref":"Formulae.html#evaluate/3"},{"type":"function","title":"Examples - Formulae.evaluate/3","doc":"iex> Formulae.eval(\"3 > 2\")\n    true\n\n    iex> Formulae.eval(\"3 < 2\")\n    false\n\n    iex> Formulae.eval(\"a < 2\", a: 1)\n    true\n\n    iex> Formulae.eval(\"a > 2\", a: 1)\n    false\n\n    iex> Formulae.eval(\"a < 2\", [])\n    {:error, {:missing_arguments, [given_keys: [], expected_keys: [:a]]}}\n\n    iex> Formulae.eval!(\"a < 2\", [])\n    ** (Formulae.RunnerError) Formula ~F[a < 2] failed to run (compile): [:missing_arguments], wrong or incomplete evaluator call: [given_keys: [], expected_keys: [:a]].\n\n    iex> Formulae.eval(\"a + 2 == 3\", a: 1)\n    true\n\n    iex> Formulae.eval(\"a + 2 == 3\", a: 2)\n    false\n\n    iex> Formulae.eval(~S|a == \"3\"|, a: \"3\")\n    true\n\n    iex> Formulae.eval(~S|a == \"3\"|, a: 3)\n    false\n\n    iex> Formulae.eval(~S|a == \"3\"|, a: \"hello\")\n    false\n\n    iex> Formulae.eval(\"a + 2 == 3\", a: 2)\n    false\n\n    iex> Formulae.eval(~S|a == \"3\"|, a: \"3\")\n    true\n\n    iex> Formulae.eval(\"a_b_c_490000 > 2\", a_b_c_490000: 3)\n    true","ref":"Formulae.html#evaluate/3-examples"},{"type":"function","title":"Formulae.formulas/1","doc":"Lists all the compiled formulas.","ref":"Formulae.html#formulas/1"},{"type":"function","title":"Formulae.normalize/1","doc":"Returns a normalized representation for the formula given.","ref":"Formulae.html#normalize/1"},{"type":"function","title":"Formulae.permutations/2","doc":"Generated clauses for `n ∈ [1..12]` to be used with dynamic number","ref":"Formulae.html#permutations/2"},{"type":"function","title":"Formulae.purge/2","doc":"Purges and discards the module for the formula given (if exists.)","ref":"Formulae.html#purge/2"},{"type":"function","title":"Formulae.unit/2","doc":"Produces the normalized representation of formula. If the _rho_ is\nan instance of [`Integer`](http://elixir-lang.org/docs/stable/elixir/Integer.html#content)\nor [`Float`](http://elixir-lang.org/docs/stable/elixir/Float.html#content),\nit’s left intact, otherwise it’s moved to the left side with negation.","ref":"Formulae.html#unit/2"},{"type":"function","title":"Examples - Formulae.unit/2","doc":"iex > Formulae.unit(\"3 > 2\")\n    {\"3 > 2\", {:>, [], [3, 2]}}\n\n    iex > Formulae.unit(\"3 - a > 2\")\n    {\"3 - a > 2\", {:>, [], [{:-, [line: 1], [3, {:a, [line: 1], nil}]}, 2]}}\n\n    iex > Formulae.unit(\"3 > A + 2\")\n    {\"3 > a + 2\",\n      {:>, [],\n        [{:-, [context: Formulae, import: Kernel],\n          [3, {:+, [line: 1], [{:a, [line: 1], nil}, 2]}]}, 0]}}\n\n    iex > Formulae.unit(\"3 >= a + 2\")\n    {\"3 >= a + 2\",\n      {:>=, [],\n        [{:-, [context: Formulae, import: Kernel],\n          [3, {:+, [line: 1], [{:a, [line: 1], nil}, 2]}]}, 0]}}\n\n    iex > Formulae.unit(\"3 a > A + 2\")\n    ** (Formulae.SyntaxError) Formula [3 a > A + 2] syntax is incorrect (parsing): syntax error before: “a”.\n\n    iex > Formulae.unit(\"a + 2 = 3\")\n    {\"a + 2 = 3\", {:==, [], [{:+, [line: 1], [{:a, [line: 1], nil}, 2]}, 3]}}\n\n    iex > Formulae.unit(~S|A = \"3\"|)\n    {\"a = \\\"3\\\"\", {:==, [], [{:a, [line: 1], nil}, \"3\"]}}","ref":"Formulae.html#unit/2-examples"},{"type":"type","title":"Formulae.t/0","doc":"The formulae is internally represented as struct, exposing the original\nbinary representing the formula, AST, the module this formula was compiled\ninto, variables (bindings) this formula has _and_ the evaluator, which is\nthe function of arity one, accepting the bindings as a keyword list and\nreturning the result of this formula application.","ref":"Formulae.html#t:t/0"},{"type":"module","title":"Formulae.Combinators","doc":"Functions to calculate all the combinations, permutations and\nrepeated permutations of the list given.","ref":"Formulae.Combinators.html"},{"type":"module","title":"Examples - Formulae.Combinators","doc":"iex> Formulae.Combinators.combinations(~w|a b c d|a, 2)\n    [[:a, :b], [:a, :c], [:a, :d], [:b, :c], [:b, :d], [:c, :d]]\n    iex> Formulae.Combinators.permutations(~w|a b c d|a, 2)\n    [[:a, :b], [:a, :c], [:a, :d], [:b, :a], [:b, :c], [:b, :d],\n     [:c, :a], [:c, :b], [:c, :d], [:d, :a], [:d, :b], [:d, :c]]\n    iex> Formulae.Combinators.repeated_permutations(~w|a b c d|a, 2)\n    [[:a, :a], [:a, :b], [:a, :c], [:a, :d], [:b, :a], [:b, :b],\n     [:b, :c], [:b, :d], [:c, :a], [:c, :b], [:c, :c], [:c, :d],\n     [:d, :a], [:d, :b], [:d, :c], [:d, :d]]\n\n**NB** this functions should not be used for relatively big `n` because\nthey perform greedy evaluation. See to `Formulae.Combinators.Stream`\nfor lazy analogues returning streams.","ref":"Formulae.Combinators.html#module-examples"},{"type":"module","title":"Dynamic numbers - Formulae.Combinators","doc":"To use combinations and permutations with dynamic number `n`, use\n`Formulae.combinations/2` and `Formulae.permutations/2`, generated\nin the compile time. By default, there are `42` clauses for combinations\nand `12` for `permutations` are generated. They might be changed in `config`,\nkeys `:formulae, :max_combinations` and `:formulae, :max_permutations`. To\nsuppress generation, use `:formulae, :generate_combinators, false`.\n\n    iex> with n <- 2, do: Formulae.combinations(~w|a b c d|a, n)\n    [[:a, :b], [:a, :c], [:a, :d], [:b, :c], [:b, :d], [:c, :d]]","ref":"Formulae.Combinators.html#module-dynamic-numbers"},{"type":"macro","title":"Formulae.Combinators.combinations/2","doc":"Calculates all combinations of the list, given as the first parameter","ref":"Formulae.Combinators.html#combinations/2"},{"type":"macro","title":"Formulae.Combinators.permutations/2","doc":"Calculates all permutations of the list, given as the first parameter","ref":"Formulae.Combinators.html#permutations/2"},{"type":"macro","title":"Formulae.Combinators.repeated_permutations/2","doc":"Calculates all repeated permutations of the list, given as the first parameter","ref":"Formulae.Combinators.html#repeated_permutations/2"},{"type":"module","title":"Formulae.Combinators.Stream","doc":"Helper to calculate all the combinations / permutations of the enumerable given.\n\nSimilar to `Formulae.Combinators` but returns a stream.\n\n_Examples_\n\n    iex> ~w|a b c d|a\n    ...> |> Formulae.Combinators.Stream.combinations(2)\n    ...> |> elem(0)\n    ...> |> Enum.to_list()\n    [[:a, :b], [:a, :c], [:a, :d], [:b, :c], [:b, :d], [:c, :d]]\n    iex> ~w|a b c d|a\n    ...> |> Formulae.Combinators.Stream.permutations(2)\n    ...> |> elem(0)\n    ...> |> Enum.to_list()\n    [[:a, :b], [:a, :c], [:a, :d], [:b, :a], [:b, :c], [:b, :d],\n     [:c, :a], [:c, :b], [:c, :d], [:d, :a], [:d, :b], [:d, :c]]\n    iex> ~w|a b c d|a\n    ...> |> Formulae.Combinators.Stream.repeated_permutations(2)\n    ...> |> elem(0)\n    ...> |> Enum.to_list()\n    [[:a, :a], [:a, :b], [:a, :c], [:a, :d], [:b, :a], [:b, :b],\n     [:b, :c], [:b, :d], [:c, :a], [:c, :b], [:c, :c], [:c, :d],\n     [:d, :a], [:d, :b], [:d, :c], [:d, :d]]\n\n    iex> (for c <- ?a..?z, do: < >)\n    ...> |> Formulae.Combinators.Stream.combinations(12)\n    ...> |> elem(0)\n    ...> |> Stream.take_every(26)\n    ...> |> Enum.take(2)\n    [[\"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"k\", \"l\"],\n     [\"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"l\", \"x\"]]\n    iex> l = for c <- ?a..?z, do: < >\n    iex> with {stream, :ok} <- Formulae.Combinators.Stream.permutations(l, 12),\n    ...>   do: stream |> Stream.take_every(26) |> Enum.take(2)\n    [[\"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"k\", \"l\"],\n     [\"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"l\", \"w\"]]\n    iex> (for c <- ?a..?z, do: < >)\n    ...> |> Formulae.Combinators.Stream.repeated_permutations(12)\n    ...> |> elem(0)\n    ...> |> Enum.take(5)\n    [[\"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\"],\n     [\"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"b\"],\n     [\"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"c\"],\n     [\"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"d\"],\n     [\"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"a\", \"e\"]]","ref":"Formulae.Combinators.Stream.html"},{"type":"macro","title":"Formulae.Combinators.Stream.combinations/2","doc":"Lazily calculates combinations of the list, given as the first parameter,\nreturns `{%Stream{}, :ok}` tuple.\n\nSee `Formulae.Combinators.combinations/2` for greedy version.","ref":"Formulae.Combinators.Stream.html#combinations/2"},{"type":"macro","title":"Formulae.Combinators.Stream.permutations/2","doc":"Lazily calculates permutations of the list, given as the first parameter,\nreturns `{%Stream{}, :ok}` tuple.\n\nSee `Formulae.Combinators.permutations/2` for greedy version.","ref":"Formulae.Combinators.Stream.html#permutations/2"},{"type":"macro","title":"Formulae.Combinators.Stream.repeated_permutations/2","doc":"Lazily calculates repeated permutations of the list, given as the first parameter,\nreturns `{%Stream{}, :ok}` tuple.\n\nSee `Formulae.Combinators.repeated_permutations/2` for greedy version.","ref":"Formulae.Combinators.Stream.html#repeated_permutations/2"},{"type":"module","title":"Formulae.Compiler","doc":"The process to be spawned to compile `Formulae` instances without clashes.\n\n_Elixir_ is great in parallel compilation, and in highly concurrent environment\nin-place compilation of the same formulas might result in clashes.\n\nIf your application already uses `Finitomata`, compiler would be created as\nan instance of it, simply launch it as\n\n```elixir\nFinitomata.start_fsm YOUR_FINITOMATA_SUP, Formulae.Compiler, \"Compiler\", nil\n```\n\notherwise it’ll be compiled into a regular `GenServer` module exporting\n`start_link/0`.\n\nOnce started, one might simply use `Compiler.compile(formula, options)`\nand `Compiler.eval(formula, options)`.","ref":"Formulae.Compiler.html"},{"type":"function","title":"Formulae.Compiler.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Formulae.Compiler.html#child_spec/1"},{"type":"function","title":"Formulae.Compiler.compile/4","doc":"Compiles a formula if it has not yet been compiled, otherwise it’s a noop.","ref":"Formulae.Compiler.html#compile/4"},{"type":"function","title":"Formulae.Compiler.eval/4","doc":"Evaluates a formula. If it hsa not been compiled yet,\nsinchronously compiles it first.","ref":"Formulae.Compiler.html#eval/4"},{"type":"function","title":"Formulae.Compiler.formulas/2","doc":"Returns all the compiled `Formulae` modules known to the system.","ref":"Formulae.Compiler.html#formulas/2"},{"type":"function","title":"Formulae.Compiler.start_link/1","doc":"","ref":"Formulae.Compiler.html#start_link/1"},{"type":"type","title":"Formulae.Compiler.t/0","doc":"","ref":"Formulae.Compiler.html#t:t/0"},{"type":"module","title":"Formulae.Sigils","doc":"Handles the sigil `~F` for `Formulae`.\n\nIt returns a compiled `Formulae` instance.","ref":"Formulae.Sigils.html"},{"type":"module","title":"Examples - Formulae.Sigils","doc":"iex> import Formulae.Sigils\n    iex> ~F[x / y > 42]\n    %Formulae{\n      ast: Code.string_to_quoted!(\"x / y > 42\"),\n      eval: &:\"Elixir.Formulae.x ÷ y > 42\".eval/1,\n      formula: \"x / y > 42\",\n      guard: nil,\n      module: :\"Elixir.Formulae.x ÷ y > 42\",\n      variables: [:x, :y],\n      options: [defaults: [], unimports: [], alias: nil, evaluator: :function, imports: []]\n    }\n    iex> ~F[x == y]ga\n    %Formulae{\n      ast: Code.string_to_quoted!(\"x == y\"),\n      eval: &:\"Elixir.Formulae.x == y\".eval/1,\n      formula: \"x == y\",\n      guard: {\n        :defguard,\n        Keyword.update!(elem(quote(generated: true, do: defguard(bar)), 1), :context, fn _ -> Formulae end),\n        [{:when, [generated: true], [{:guard, [generated: true], [{:x, [], nil}, {:y, [], nil}]}, {:==, [line: 1], [{:x, [line: 1], nil}, {:y, [line: 1], nil}]}]}]\n      },\n      module: :\"Elixir.Formulae.x == y\",\n      variables: [:x, :y],\n      options: [defaults: [], unimports: [], alias: nil, evaluator: :guard, imports: [:...]]\n    }","ref":"Formulae.Sigils.html#module-examples"},{"type":"macro","title":"Formulae.Sigils.sigil_F/2","doc":"Convenience sigil to declare `Formulae`. Maps to `Formulae.compile/2` with `imports: :none`\n  (or with `imports: :all` if the `a` modifier is given.)","ref":"Formulae.Sigils.html#sigil_F/2"},{"type":"exception","title":"Formulae.RunnerError","doc":"","ref":"Formulae.RunnerError.html"},{"type":"exception","title":"Formulae.SyntaxError","doc":"","ref":"Formulae.SyntaxError.html"},{"type":"extras","title":"Formulae","doc":"# Formulae    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/am-kantox/formulae/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/am-kantox/formulae/workflows/Dialyzer/badge.svg)\n\n**Nifty precompiled formulae handling**","ref":"readme.html"},{"type":"extras","title":"Installation - Formulae","doc":"```elixir\ndef deps do\n  [{:formulae, \"~> 0.11\"}]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Using - Formulae","doc":"```elixir\niex|1 ▸ f = Formulae.compile(\"rem(a, 5) - b == 0\")\n#ℱ<[\n  sigil: \"~F[rem(a, 5) - b == 0]\",\n  eval: &:\"Elixir.Formulae.rem(a, 5) - b == 0\".eval/1,\n  formula: \"rem(a, 5) - b == 0\",\n  guard: nil,\n  module: :\"Elixir.Formulae.rem(a, 5) - b == 0\",\n  variables: [:a, :b]\n]>\niex|2 ▸ f.eval.(a: 11, b: 1)\ntrue\niex|3 ▸ f.variables\n[:a, :b]\niex|4 ▸ f.module\n:\"Elixir.Formulae.rem(a, 5) - b == 0\"\n```","ref":"readme.html#using"},{"type":"extras","title":"Changelog - Formulae","doc":"- **`0.17.0`** [UPD] Allow explicit black list for what is imported from `Kernel` via `unimports:`\n- **`0.16.0`** [FIX] Allow formulas longer than 255 bytes (module name becomes a hash)\n- **`0.14.0`** [UPD] Fully optional `Finitomata`, configurable through `config :formulae, compiler: :finitomata`\n- **`0.13.0`** [UPD] Elixir v1.16, modern libs\n- **`0.12.0`** [ENH] `Formulae.Compiler` to avoid compilation glitches in highly concurrent environment\n- **`0.11.7`** make dialyzer happy with new `:imports` formats\n- **`0.11.6`** improve syntax for imports with arguments, accept `{:math, only: [pi: 0]}`\n- **`0.11.5`** [FIX] elixir nested aliases + [FEAT] accept `except:` and `only:` for imports\n- **`0.11.4`** [FIX] reattempt to create module if not existing on `eval/3` call\n- **`0.11.2`** [FIX] restrict `apply/3`, `spawn/3`, and `import/2`\n- **`0.11.1`** [FIX] `Formulae.formulas/1` + aliases are handled more naturally\n- **`0.11.0`** `v1.0.0` pre-release\n  - `options` in a call to `Formulae.compile/2`\n  - accurate validation of module existence\n  - default arguments\n  - aliases\n  - imports\n- **`0.10.0`**\n  - `Inspect` and `String.Chars` protocols implementation\n  - `Formulae.Sigils.sigil_F/2` aka `~F[x > y]`\n- **`0.9.0`**\n  - `options: [eval: :guard, alias: SomeModName]`\n  - fixes for modern _Elixir_\n  - optimizations in module generation\n  - benchmarks\n- **`0.8.0`** — generate combinators functions in `Formulae` module\n\n## [Documentation](https://hexdocs.pm/formulae)","ref":"readme.html#changelog"}],"proglang":"elixir","producer":{"name":"ex_doc","version":"0.35.1"},"content_type":"text/markdown"}